%% A comparison of numerical methods for the solution of continuous-time DSGE models
%  Author: Juan Carlos Parra-Alvarez
%  Aarhus University
%  September 2013
%  Updated -> February 2016
%
%  Linear-Quadratic approximation: Investment
%
%  Description of file: mainLQ_bench_i.m computes the approximated value
%  and policy funtions for investment and labor supply for the RBC model 
%  without parametric restrictions. Consumption is later recovered from
%  the budget constraint.
%
%  To run the code it is necessary to have CompEcon toolbox installed in
%  the root of Matlab. It is also necessary to have the function "care" to
%  solve continuous-time algebraic Riccati equations.
%
%  To load the steady state of the model, you need to run first the file
%  SteadyState.m found in the Steady State folder. Obs: Check that 
%  the model parameters section coincidence between files. 
%
%  The external function model_i.m defines the objective, drift and
%  diffusion functions of the model.   

%% ---------------------------------------------------------------------- %
%                             House keeping                               %
%-------------------------------------------------------------------------%
clear all
clc
close all

%% ---------------------------------------------------------------------- %
%                            Model Parameters                             %
%-------------------------------------------------------------------------%
global rho gamma psi alpha delta kappa omega sigmaK eta;

rho    = 0.0105;
gamma  = 2;
psi    = 1.8011;
alpha  = 0.4;
delta  = 0.0196;
kappa  = 0.05;
omega  = 1;
sigmaK = 0.000; 
eta    = 0.007;
sdA    = sqrt(omega*(eta^2/(2*kappa))); % Unconditional s.d. of TFP 

%% ---------------------------------------------------------------------- %
%                    Deterministic Steady State                           %
%-------------------------------------------------------------------------%
% Obs: Check that the model parameters section coincidence between files.
load('/Users/Juan.Carlos.Parra.Alvarez/Dropbox/Submission_MD/OnlineCodes/SteadyState/steadystate.mat');
Ass  = ss_sol(1);
Kss  = ss_sol(2);
lss  = ss_sol(3);
css  = ss_sol(4);
vss  = ss_sol(5);
vkss = ss_sol(6);
vAss = ss_sol(7);
rss  = ss_sol(8);
wss  = ss_sol(9);
iss  = Ass*Kss^alpha*lss^(1-alpha) - css;
clear('ss_sol');

s_ss = [Kss, Ass];   % Steady state value for the state variable (s)
c_ss = [iss, lss];   % Steady state value for the control variables (c)
ds = length(s_ss); 
dc = length(c_ss); 

%% ---------------------------------------------------------------------- %
%                        Numerical approximation                          %
%-------------------------------------------------------------------------%
tic;

%-------------------------------------------------------------------------%
%                   Function approximation parameters                     %
%-------------------------------------------------------------------------%
ns   = [301 301];                      % Number of grid points
smin = [0.5*Kss, Ass - 10*sdA/2];      % Lower bound on the state variable 
smax = [1.5*Kss, Ass + 10*sdA/2];      % Upper bound on the state variable 
for i=1:ds
    scoord{i} = linspace(smin(i),smax(i),ns(i))'; %Evaluation points at each state.
    scoord{i} = scoord{i} - s_ss(i);
end
Shat = gridmake(scoord);               % State variable (uniform) grid

%-------------------------------------------------------------------------%
%                       Definition of the model                           %
%-------------------------------------------------------------------------%
model.func   = 'model_i';
model.params = {rho,gamma,psi,alpha,delta,sigmaK,eta,kappa,omega};
scfile       = model.func;
params       = model.params;

%-------------------------------------------------------------------------%
%          Objective function, drift function and discount factor         %
%-------------------------------------------------------------------------%
pi   = feval(scfile,'f',s_ss,c_ss,params{:});
mu   = feval(scfile,'g',s_ss,c_ss,params{:});
sig_ = reshape(feval(scfile,'sigma',s_ss,c_ss,params{:}),2,2);

%-------------------------------------------------------------------------%
%                             Derivatives                                 %
%-------------------------------------------------------------------------%
%                          With respect to c                              %
%-------------------------------------------------------------------------%
pi_c  = fjac(scfile ,3,'f',s_ss,c_ss,params{:});
pi_cc = fhess(scfile,3,'f',s_ss,c_ss,params{:});
mu_c  = fjac(scfile ,3,'g',s_ss,c_ss,params{:});
mu_cc = fhess(scfile,3,'g',s_ss,c_ss,params{:});

%-------------------------------------------------------------------------%
%                          With respect to s                              %
%-------------------------------------------------------------------------%
pi_s  = fjac(scfile ,2,'f',s_ss,c_ss,params{:});
pi_ss = fhess(scfile,2,'f',s_ss,c_ss,params{:});
mu_s  = fjac(scfile ,2,'g',s_ss,c_ss,params{:});
mu_ss = fhess(scfile,2,'g',s_ss,c_ss,params{:});

%-------------------------------------------------------------------------%
%                          Cross-derivatives                              %
%-------------------------------------------------------------------------%
if sum(pi_s)==0
    pi_sc = zeros(ds,dc);
else
    syms At kap lab inv
    u = ((At*kap^alpha*lab^(1-alpha) - inv)*(1-lab)^psi)^(1-gamma)/(1-gamma);
    pi_ki = diff(diff(u,kap),inv);
    pi_ai = diff(diff(u,At),inv);
    pi_kl = diff(diff(u,kap),lab);
    pi_al = diff(diff(u,At),lab);
    h     = matlabFunction([pi_ki,pi_kl; pi_ai, pi_al],'file','d_sc'); %The input variables of the new function are organized alphabetically
    pi_sc = double(d_sc(Ass,iss,Kss,lss));
end
pi_cs = pi_sc';

%-------------------------------------------------------------------------%
%             Expansion of drift functions to include constant            %
%-------------------------------------------------------------------------%
ds_                  = ds + 1;
mu_sbar              = zeros(ds_,ds_);
mu_cbar              = zeros(ds_,dc);
mu_sbar(2:end,2:end) = mu_s;
mu_cbar(2:end,:)     = mu_c;
sig_bar              = zeros(ds_,ds);
sig_bar(2:end,:)     = sig_;

%-------------------------------------------------------------------------%
%                  Quadratic representation (McCandless)                  %
%-------------------------------------------------------------------------%
R = 0.5*[2*pi pi_s; pi_s' pi_ss];
Q = 0.5*pi_cc;
W = 0.5*[pi_c; pi_sc];

%-------------------------------------------------------------------------%
%         Simplification of cross-products (Ljungqvist and Sargent)       %
%-------------------------------------------------------------------------%
Rbar    = R - (W/Q)*W';
mu_sbar = mu_sbar - (mu_cbar/Q)*W';

%-------------------------------------------------------------------------%
%                Ricatti Equation and remaining constants                 %
%-------------------------------------------------------------------------%
G = Rbar;
H = Q;
B = mu_sbar - 0.5*rho*eye(ds_);
C = mu_cbar;

% Solution to the Ricatti Equation
Lss = care(B,C,G,H);
L   = rho^(-1)*trace(sig_bar*sig_bar'*Lss);

% check solution
B'*Lss + Lss*B - ((Lss*C)/H)*(C'*Lss) + G

%-------------------------------------------------------------------------%
%                           Approximated solution                         %
%-------------------------------------------------------------------------%
K = scoord{1};
A = scoord{2};
Shat_ = [ones(size(Shat,1),1),Shat];

C_ss = repmat(c_ss',size(c_ss',2),size(Shat_',2));
C = C_ss -Q\(W'+mu_cbar'*Lss)*Shat_';
C = C';
C = reshape(C,[ns dc]);

% Investment approximation
i_hat = C(:,:,1);
% Labor supply approximation
l_hat = C(:,:,2);
% Consumption approximation (recovered from the budget constraint)
for i=1:ns(1)
    for j=1:ns(2)
        c_hat(i,j) = (A(j)+Ass)*(K(i)+Kss).^alpha.*l_hat(i,j).^(1-alpha) - i_hat(i,j);
    end
end
% Value function approximations
for i = 1:size(Shat_,1)
    V(i,1) = L + Shat_(i,:)*Lss*Shat_(i,:)';
end
v_hat = reshape(V,ns); 
toc;

%-------------------------------------------------------------------------%
%                   Numerical approximation errors                        %
%-------------------------------------------------------------------------%
vs_hat = reshape(2*Shat_*Lss,[ns ds_]);
v1_hat = vs_hat(:,:,1);
vk_hat = vs_hat(:,:,2);
vA_hat = vs_hat(:,:,3);
vss_hat= 2*Lss;
vkk_hat= vss_hat(2,2);
vAA_hat= vss_hat(3,3);
for i=1:ns(1)
    for j=1:ns(2)
        resid(i,j) = -rho*v_hat(i,j) + ((c_hat(i,j)*(1-l_hat(i,j))^psi)^(1-gamma))/(1-gamma) ... 
            + (A(j)*K(i)^alpha*l_hat(i,j)^(1-alpha) - delta*K(i) - c_hat(i,j))*vk_hat(i,j) ...
            + kappa*(omega-A(j))*vA_hat(i,j) + 0.5*sigmaK^2*K(i)^2*vkk_hat + 0.5*eta^2*A(j)*vAA_hat;
        resid_LQi(i,j) = log(abs(resid(i,j))+0.000000000001)/log(10);
    end
end

%-------------------------------------------------------------------------%
%                            Plot results                                 %
%-------------------------------------------------------------------------%
% Finding the position of the steady state values of the state variables
Ass_loc = 151;
Kss_loc = 151;

%-------------------------------------------------------------------------%
%                       Value and policy functions (2-D)                  %
%-------------------------------------------------------------------------%
fig = figure(1);
setmyfig_out(fig);
set(gca, 'FontSize', 14)
subplot(3,1,1)
plot(K/Kss,v_hat(:,Ass_loc))
ylabel('$V(K_{t},A^{ss})$','FontSize',16,'Interpreter','latex')
title('Value function','FontSize',16,'Interpreter','latex')
axis([-inf inf -inf inf])
grid on

subplot(3,1,2)
plot(K/Kss,c_hat(:,Ass_loc))
title('Consumption Function','FontSize',16,'Interpreter','latex')
ylabel('$C(K_{t},A^{ss})$','FontSize',16,'Interpreter','latex')
axis([-inf inf -inf inf])
grid on

subplot(3,1,3)
plot(K/Kss,l_hat(:,Ass_loc))
title('Labour Supply Function','FontSize',16,'Interpreter','latex')
ylabel('$L(K_{t},A^{ss})$','FontSize',16,'Interpreter','latex')
xlabel('$K/Kss$','FontSize',16,'Interpreter','latex')
axis([-inf inf -inf inf])
grid on

% figure(2)
% subplot(3,1,1)
% plot(A/Ass,v_hat(Kss_loc,:))
% ylabel('V(Kss,A)')
% xlabel('Productivity')
% axis([-inf inf -inf inf])
% subplot(3,1,2)
% plot(A/Ass,c_hat(Kss_loc,:))
% ylabel('C(Kss,A)')
% xlabel('Productivity')
% axis([-inf inf -inf inf])
% subplot(3,1,3)
% plot(A/Ass,l_hat(Kss_loc,:))
% ylabel('L(Kss,A)')
% xlabel('Productivity')
% axis([-inf inf -inf inf])

%-------------------------------------------------------------------------%
%                       Value and policy functions (3-D)                  %
%-------------------------------------------------------------------------%
[KX,AX] = meshgrid(K,A);
fig = figure(2);
setmyfig_out(fig);
subplot(3,1,1)
mesh(KX/Kss,AX/Ass,v_hat')
shading interp
xlabel('$K/Kss$','FontSize',16,'Interpreter','latex')
ylabel('$A/Ass$','FontSize',16,'Interpreter','latex')
zlabel('$V(K_{t},A_{t})$','FontSize',16,'Interpreter','latex')
axis([-inf inf -inf inf -inf inf])

subplot(3,1,2)
mesh(KX/Kss,AX/Ass,c_hat')
shading interp
xlabel('$K/Kss$','FontSize',16,'Interpreter','latex')
ylabel('$A/Ass$','FontSize',16,'Interpreter','latex')
zlabel('$C(K_{t},A_{t})$','FontSize',16,'Interpreter','latex')
axis([-inf inf -inf inf -inf inf])

subplot(3,1,3)
mesh(KX/Kss,AX/Ass,l_hat')
shading interp
xlabel('$K/Kss$','FontSize',16,'Interpreter','latex')
ylabel('$A/Ass$','FontSize',16,'Interpreter','latex')
zlabel('$L(K_{t},A_{t})$','FontSize',16,'Interpreter','latex')
axis([-inf inf -inf inf -inf inf])