function F=fun_proj(Phi, Phi1, Phi2, S, method, coeff)

% F=fun_proj(Phi, Phi1, Phi2, S, method, coeff): defines the projection
% equations to approximate the value function and the policy function for
% labor supply. 
% Inputs:
% Phi    : Level of Chebyshev polynomials
% Phi1   : First derivative of Chebyshev polynomials
% Phi2   : Second derivative of Chebyshev polynomials
% S      : Grid points for the state variable
% method : Projection method
% coeff  : coefficients of the approximation
%
% Output:
% F      : Value of the objective function which depends on the method
%          used.
%
%  Author: Juan Carlos Parra-Alvarez
%  Aarhus University
%  September 2013

%% ---------------------------------------------------------------------- %
%                            Model Parameters                             %
%-------------------------------------------------------------------------%
global rho gamma psi alpha delta kappa omega sigma eta;

n = size(coeff,1);         % Total number of coefficients to be approximated   
K = S(:,1);                % Grid points for the state variable K
A = S(:,2);                % Grid points for the state variable A
 
a = coeff(1:n/2);          % Coefficients to approximate value function
b = coeff((n/2)+1:end);    % Coefficients to approximate Labor supply function

% Value function approximations
V   = Phi*a;
Vk  = Phi1{1}*a;
VA  = Phi1{2}*a;
Vkk = Phi2{1}*a;
VAA = Phi2{2}*a;

% Labor supply function approximations
L = Phi*b;

% Prices
r = alpha*A.*(K.^(alpha-1)).*(L.^(1-alpha));
w = (1-alpha)*A.*(K.^alpha).*(L.^(-alpha));

% Consumption function (from the F.O.C)
C = (Vk.*(1-L).^(-psi*(1-gamma))).^(-1/gamma);
    
% Hamilton-Jacobi-Bellman Equation
F1 =-rho*V + ((C.*(1-L).^psi).^(1-gamma))/(1-gamma) + ((r-delta).*K + w.*L - C).*Vk + kappa*(omega-A).*VA + 0.5*(sigma^2)*(K.^2).*Vkk + 0.5*(eta^2)*A.*VAA;
% Labor supply (F.O.C)
F2 =(1-L).^(((psi*(1-gamma)^2)/gamma)+psi*(1-gamma)-1).*Vk.^(-1/gamma)-((1-alpha)/psi).*A.*K.^alpha.*L.^(-alpha);

R = [F1;F2];

if method == 1
    F  = R;    % Collocation: fsolve
elseif method == 2
    F = R'*R;  % Least-Squares: fmincon
end
   
end
    
    