%% A comparison of numerical methods for the solution of continuous-time DSGE models
%  Author: Juan Carlos Parra-Alvarez
%  Aarhus University
%  September 2013
%  Updated -> February 2016
%
%  Projection method: Collocation and Least Squares methods
%
%  Description of file: mainProj_closed.m computes the approximated value
%  and policy funtions for the RBC model without parametric restrictions.
%  The method uses two approximants. One for the value function and other
%  for the policy function associated with labor supply. Once both 
%  approximants are computed, the consumption function is recovered from 
%  the first order conditions of the model.
%
%  To choose the type of approximant change the value of the variable
%  method in line 113:
%                       method = 1 -> Collocation method
%                       method = 2 -> Least squares method
%
%  To run the code it is necessary to have CompEcon toolbox installed in
%  the root of Matlab.
%
%  To load the steady state of the model, you need to run first the file
%  SteadyState_closed.m found in the Steady State folder. Obs: Check that 
%  the model parameters section coincidence between files.
%
%  This code uses the true solution as initial value for the numerical
%  procedures. 
%
%  The external function fun_proj.m defines the projection the residual 
%  functions. 

%% ---------------------------------------------------------------------- %
%                             House keeping                               %
%-------------------------------------------------------------------------%
clear
clc
close all

%% ---------------------------------------------------------------------- %
%                            Model Parameters                             %
%-------------------------------------------------------------------------%
global rho gamma psi alpha delta kappa omega sigma eta;

rho    = 0.0105;
gamma  = 2;
psi    = 1.8011;
alpha  = 0.4;
delta  = 0.0196;
kappa  = 0.05;
omega  = 1;
sigma  = 0.000; 
eta    = 0.007;
sdA   = sqrt(omega*(eta^2/(2*kappa))); % Unconditional s.d. of TFP 

%% ---------------------------------------------------------------------- %
%                    Deterministic Steady State                           %
%-------------------------------------------------------------------------%
% Obs: Check that the model parameters section coincidence between files.
load('/Users/Juan.Carlos.Parra.Alvarez/Dropbox/Submission_MD/OnlineCodes/SteadyState/steadystate.mat');
Ass  = ss_sol(1);
Kss  = ss_sol(2);
lss  = ss_sol(3);
css  = ss_sol(4);
vss  = ss_sol(5);
vkss = ss_sol(6);
vAss = ss_sol(7);
rss  = ss_sol(8);
wss  = ss_sol(9);
clear('ss_sol');

%% ---------------------------------------------------------------------- %
%               Numerical approximation: Collocation method               %
%-------------------------------------------------------------------------%
tic;

%-------------------------------------------------------------------------%
%                   Function approximation parameters                     %
%-------------------------------------------------------------------------%
n      = [9,9];                              % Order of approximation (in two dimensions)
smin   = [0.5*Kss, Ass-10*sdA/2];            % Lower bound on the state variables
smax   = [1.5*Kss, Ass+10*sdA/2];            % Upper bound on the state variables 
fspace = fundefn('cheb',n,smin,smax);        % Definition of basis functions
d      = fspace.d;                           % Number of state variables

%-------------------------------------------------------------------------%
%                    Chebychev nodes in [smin, smax]                      %
%-------------------------------------------------------------------------%
snodes  = funnode(fspace);
snodes1 = gridmake(snodes);

%-------------------------------------------------------------------------%
%                      Chebyshev Basis functions                          %
%-------------------------------------------------------------------------%
bases = funbasx(fspace,snodes,[0;1;2]);
Phi   = ctbasemake([],bases,0);      % Chebyshev polynomials
Phi1  = ctbasemake([],bases,1);      % First order derivative of the Chebyshev polynomials
Phi2  = ctbasemake([],bases,2);      % Second order derivative of the Chebyshev polynomials

%-------------------------------------------------------------------------%
%              Initial guess for approximation coefficients               %
%-------------------------------------------------------------------------%
% Initial values used in the paper. 
% It loads the vector coeff of dimensions [(nx2x2)x1] with approximation coefficients.
load('/Users/Juan.Carlos.Parra.Alvarez/Dropbox/Submission_MD/OnlineCodes/BenchmarkModel/Projection/ini_vals.mat');

%-------------------------------------------------------------------------%
%                    Projection algorithm -> p(x)                         %
%-------------------------------------------------------------------------%
tol    = sqrt(eps);
algo   = 'trust-region-dogleg';
method = 1;   
switch method
    case 1 % Collocation method
        options = optimset('Display','iter','MaxIter', 100000, 'MaxFunEvals', 100000, 'TolFun', tol,'Algorithm', algo, 'Jacobian', 'off');
        coeffi       = coeff;
        coeff        = fsolve(@(coeff) fun_proj(Phi,Phi1,Phi2,snodes1,method,coeff), coeffi, options);
        options      = optimset('Display','iter','MaxIter', 50000, 'MaxFunEvals', 50000, 'TolFun', tol,'Algorithm', algo, 'Jacobian', 'off');
        [coeff,fval] = fsolve(@(coeff) fun_proj(Phi,Phi1,Phi2,snodes1,method,coeff), coeff, options);
    case 2 % Least Squares method
        options      = optimset('Display','iter','MaxIter', 100000, 'MaxFunEvals', 100000, 'TolFun', tol,'Algorithm', algo, 'Jacobian', 'off');
        coeffi       = coeff;
        [coeff,fval] = fminsearch(@(coeff) fun_proj(Phi,Phi1,Phi2,snodes1,method,coeff), coeffi, options);
end
toc;

%-------------------------------------------------------------------------%
%                            Evaluation grid                              %
%-------------------------------------------------------------------------%
l = size(coeff,1);
a = coeff(1:l/2);
b = coeff((l/2)+1:end);

nresK = 301; %Number of extra evaluation points in the K-lattice
nresA = 301; %Number of extra evaluation points in the A-lattice
n(1)  = nresK;
n(2)  = nresA;
for i=1:d
    scoord{i} = linspace(fspace.a(i),fspace.b(i),n(i))'; %Evaluation points at each state.
end
S = gridmake(scoord);

% Basis functions on the evaluation grid
bases= funbasx(fspace,scoord,[0;1;2]);
Phi  = ctbasemake([],bases,0);
Phi1 = ctbasemake([],bases,1);
Phi2 = ctbasemake([],bases,2);

% Value function approximations
V   = Phi*a;
Vk  = Phi1{1}*a;
VA  = Phi1{2}*a;
Vkk = Phi2{1}*a;
VAA = Phi2{2}*a;

% Labor supply approximation
L = Phi*b;

% Optimal prices
r = alpha*S(:,2).*(S(:,1).^(alpha-1)).*(L.^(1-alpha));
w = (1-alpha)*S(:,2).*(S(:,1).^alpha).*(L.^(-alpha));

% Consumption approximation
C = (Vk.*(1-L).^(-psi*(1-gamma))).^(-1/gamma);

V   = reshape(V,[n 1]);
Vk  = reshape(Vk,[n 1]);
Vkk = reshape(Vkk,[n 1]);
VA  = reshape(VA,[n 1]);
VAA = reshape(VAA,[n 1]);
C   = reshape(C,[n size(C,2) 1]);
L   = reshape(L,[n size(L,2) 1]);

%-------------------------------------------------------------------------%
%                   Numerical approximation errors                        %
%-------------------------------------------------------------------------%
K = scoord{1};
A = scoord{2};

if method == 1
    for i=1:n(1)
        for j=1:n(2)
            resid(i,j) = -rho*V(i,j) + ((C(i,j)*(1-L(i,j))^psi)^(1-gamma))/(1-gamma) + (A(j)*K(i)^alpha*L(i,j)^(1-alpha) - delta*K(i) - C(i,j))*Vk(i,j) ...
                + kappa*(omega-A(j))*VA(i,j) + 0.5*sigma^2*K(i)^2*Vkk(i,j) + 0.5*eta^2*A(j)*VAA(i,j);
            resid_col(i,j) = log(abs(resid(i,j))+0.000000000001)/log(10);
        end
    end
else
    for i=1:n(1)
        for j=1:n(2)
            resid(i,j) = -rho*V(i,j) + ((C(i,j)*(1-L(i,j))^psi)^(1-gamma))/(1-gamma) + (A(j)*K(i)^alpha*L(i,j)^(1-alpha) - delta*K(i) - C(i,j))*Vk(i,j) ...
                + kappa*(omega-A(j))*VA(i,j) + 0.5*sigma^2*K(i)^2*Vkk(i,j) + 0.5*eta^2*A(j)*VAA(i,j);
            resid_ols(i,j) = log(abs(resid(i,j))+0.000000000001)/log(10);
        end
    end
end

%-------------------------------------------------------------------------%
%                            Plot results                                 %
%-------------------------------------------------------------------------%
% Finding the position of the steady state values of the state variables
Ass_loc = find(A==Ass);
Kss_loc = find(K==Kss);
 
%-------------------------------------------------------------------------%
%                       Value and policy functions (2-D)                  %
%-------------------------------------------------------------------------%
fig = figure(1);
setmyfig_out(fig);
set(gca, 'FontSize', 14);
subplot(3,1,1)
plot(K/Kss,V(:,Ass_loc))
ylabel('$V(K_{t},A^{ss})$','FontSize',16,'Interpreter','latex')
title('Value function','FontSize',16,'Interpreter','latex')
axis([-inf inf -inf inf])
grid on

subplot(3,1,2)
plot(K/Kss,C(:,Ass_loc))
title('Consumption Function','FontSize',16,'Interpreter','latex')
ylabel('$C(K_{t},A^{ss})$','FontSize',16,'Interpreter','latex')
axis([-inf inf -inf inf])
grid on

subplot(3,1,3)
plot(K/Kss,L(:,Ass_loc))
title('Labour Supply Function','FontSize',16,'Interpreter','latex')
ylabel('$L(K_{t},A^{ss})$','FontSize',16,'Interpreter','latex')
xlabel('$K/Kss$','FontSize',16,'Interpreter','latex')
axis([-inf inf -inf inf])
grid on

%-------------------------------------------------------------------------%
%                       Value and policy functions (3-D)                  %
%-------------------------------------------------------------------------%
[KX,AX] = meshgrid(K,A);
fig = figure(2);
setmyfig_out(fig);
subplot(3,1,1)
set(gca, 'FontSize', 14);
surf(KX/Kss,AX/Ass,V')
shading interp
xlabel('$K/Kss$','FontSize',16,'Interpreter','latex')
ylabel('$A/Ass$','FontSize',16,'Interpreter','latex')
zlabel('$V(K_{t},A_{t})$','FontSize',16,'Interpreter','latex')
axis([-inf inf -inf inf -inf inf])

subplot(3,1,2)
surf(KX/Kss,AX/Ass,C')
shading interp
xlabel('$K/Kss$','FontSize',16,'Interpreter','latex')
ylabel('$A/Ass$','FontSize',16,'Interpreter','latex')
zlabel('$C(K_{t},A_{t})$','FontSize',16,'Interpreter','latex')
axis([-inf inf -inf inf -inf inf])

subplot(3,1,3)
surf(KX/Kss,AX/Ass,L')
shading interp
xlabel('$K/Kss$','FontSize',16,'Interpreter','latex')
ylabel('$A/Ass$','FontSize',16,'Interpreter','latex')
zlabel('$L(K_{t},A_{t})$','FontSize',16,'Interpreter','latex')
axis([-inf inf -inf inf -inf inf])
