%% A comparison of numerical methods for the solution of continuous-time DSGE models
%  Author: Juan Carlos Parra-Alvarez
%  Aarhus University
%  September 2013
%  Updated -> February 2016
%
%  Linear-Quadratic approximation: Consumption 
%
%  Description of file: mainLQ_closed_c.m computes the approximated value
%  and policy funtions for consumption and labor supply for the RBC model 
%  under proposition 3.1 in the paper. 
%
%  To run the code it is necessary to have CompEcon toolbox installed in
%  the root of Matlab. It is also necessary to have the function "care" to
%  solve continuous-time algebraic Riccati equations.
%
%  To load the steady state of the model, you need to run first the file
%  SteadyState_closed.m found in the Steady State folder. Obs: Check that 
%  the model parameters section coincidence between files. 
%
%  The external function model_c.m defines the objective, drift and
%  diffusion functions of the model.   

%% ---------------------------------------------------------------------- %
%                             House keeping                               %
%-------------------------------------------------------------------------%
clear all
clc
close all

%% ---------------------------------------------------------------------- %
%                            Model Parameters                             %
%-------------------------------------------------------------------------%
global rho gamma psi alpha delta sigmaK eta A;

A      = 1;
gamma  = 3.85;
psi    = 1.8011;
delta  = 0.0196;
alpha  = 0.4;
sigmaK = 0.001;
rho    = -(1-alpha*gamma)*(delta+0.5*alpha*gamma*sigmaK^2);

%% ---------------------------------------------------------------------- %
%                    Deterministic Steady State                           %
%-------------------------------------------------------------------------%
% Obs: Check that the model parameters section coincidence between files.
load('/Users/Juan.Carlos.Parra.Alvarez/Dropbox/Submission_MD/OnlineCodes/SteadyState/steadystate_closed.mat');
Kss  = ss_sol(1);
lss  = ss_sol(2);
css  = ss_sol(3);
vss  = ss_sol(4);
vkss = ss_sol(5);
rss  = ss_sol(6);
wss  = ss_sol(7);
clear('ss_sol');

s_ss = Kss;           % Steady state value for the state variable (s)
c_ss = [css, lss];    % Steady state value for the control variables (c)
ds   = length(s_ss); 
dc   = length(c_ss); 

%% ---------------------------------------------------------------------- %
%                        Numerical approximation                          %
%-------------------------------------------------------------------------%
tic;

%-------------------------------------------------------------------------%
%                   Function approximation parameters                     %
%-------------------------------------------------------------------------%
ns   = 1000;               % Number of grid points
smin = 0.1*Kss;            % Lower bound on the state variable 
smax = 1.5*Kss;            % Upper bound on the state variable 
for i=1:ds
    scoord{i} = linspace(smin(i),smax(i),ns(i))'; %Evaluation points at each state.
    scoord{i} = scoord{i} - s_ss(i);
end
Shat = gridmake(scoord);    % State variable (uniform) grid

%-------------------------------------------------------------------------%
%                       Definition of the model                           %
%-------------------------------------------------------------------------%
model.func   = 'model_c';
model.params = {rho gamma psi alpha delta sigmaK eta A};
scfile       = model.func;
params       = model.params;

%-------------------------------------------------------------------------%
%          Objective function, drift function and discount factor         %
%-------------------------------------------------------------------------%
pi   = feval(scfile,'f',s_ss,c_ss,params{:});
mu   = feval(scfile,'g',s_ss,c_ss,params{:});
sig_ = reshape(feval(scfile,'sigma',s_ss,c_ss,params{:}),1,1);

%-------------------------------------------------------------------------%
%                             Derivatives                                 %
%-------------------------------------------------------------------------%
%                          With respect to c                              %
%-------------------------------------------------------------------------%
pi_c  = fjac(scfile ,3,'f',s_ss,c_ss,params{:});
pi_cc = fhess(scfile,3,'f',s_ss,c_ss,params{:});
mu_c  = fjac(scfile ,3,'g',s_ss,c_ss,params{:});
mu_cc = fhess(scfile,3,'g',s_ss,c_ss,params{:});

%-------------------------------------------------------------------------%
%                          With respect to s                              %
%-------------------------------------------------------------------------%
pi_s  = fjac(scfile ,2,'f',s_ss,c_ss,params{:});
pi_ss = fhess(scfile,2,'f',s_ss,c_ss,params{:});
mu_s  = fjac(scfile ,2,'g',s_ss,c_ss,params{:});
mu_ss = fhess(scfile,2,'g',s_ss,c_ss,params{:});

%-------------------------------------------------------------------------%
%                          Cross-derivatives                              %
%-------------------------------------------------------------------------%
if sum(pi_s)==0
    pi_sc = zeros(ds,dc);
else
    %not yet supported, please provide analytical derivatives
end
pi_cs = pi_sc';

%-------------------------------------------------------------------------%
%             Expansion of drift functions to include constant            %
%-------------------------------------------------------------------------%
ds_                  = ds + 1;
mu_sbar              = zeros(ds_,ds_);
mu_cbar              = zeros(ds_,dc);
mu_sbar(2:end,2:end) = mu_s;
mu_cbar(2:end,:)     = mu_c;
sig_bar              = zeros(ds_,ds);
sig_bar(2:end,:)     = sig_;

%-------------------------------------------------------------------------%
%                  Quadratic representation (McCandless)                  %
%-------------------------------------------------------------------------%
R = 0.5*[2*pi pi_s; pi_s' pi_ss];
Q = 0.5*pi_cc;
W = 0.5*[pi_c; pi_sc];

%-------------------------------------------------------------------------%
%         Simplification of cross-products (Ljungqvist and Sargent)       %
%-------------------------------------------------------------------------%
Rbar    = R - W*inv(Q)*W';
mu_sbar = mu_sbar - mu_cbar*inv(Q)*W';

%-------------------------------------------------------------------------%
%                Ricatti Equation and remaining constants                 %
%-------------------------------------------------------------------------%
G = Rbar;
H = Q;
B = mu_sbar - 0.5*rho*eye(ds_);
C = mu_cbar;

% Solution to the Ricatti Equation
Lss = care(B,C,G,H);
L   = rho^(-1)*trace(sig_bar*sig_bar'*Lss);

% check solution
B'*Lss + Lss*B - (Lss*C)*inv(H)*(C'*Lss) + G

%-------------------------------------------------------------------------%
%                           Approximated solution                         %
%-------------------------------------------------------------------------%
Shat_ = [ones(size(Shat,1),1),Shat];

C_ss  = repmat(c_ss',size(c_ss',2),size(Shat_',2));
C     = C_ss -inv(Q)*(W'+mu_cbar'*Lss)*Shat_';
C     = C';

% Consumption approximation
c_hat = C(:,1);
% Labor supply approximation
l_hat = C(:,2);

for i = 1:size(Shat_,1)
    V(i,1) = L + Shat_(i,:)*Lss*Shat_(i,:)';
end
% Value function approximation
v_hat = reshape(V,ns,ds); 
toc;

%-------------------------------------------------------------------------%
%                  Closed form solution (proposition 3.1)                 %
%-------------------------------------------------------------------------%
K  = scoord{1};
Lt = gamma*(1-alpha)/(gamma*(1-alpha)-psi*(1-gamma));
s  = 1-(1-alpha)*(1-Lt)/(psi*Lt);
C1 = -((1-s)^(1-gamma)*A^(1-gamma)*Lt^((1-alpha)*(1-gamma))*(1-Lt)^(psi*(1-gamma)))/((1-gamma)*A*Lt^(1-alpha)-(1-gamma)*(1-s)*A*Lt^(1-alpha));
Vt = C1*(((K+Kss).^(1-alpha*gamma))/(1-alpha*gamma));
Ct = (1-s)*A*(K+Kss).^alpha*Lt^(1-alpha);
Lt = repmat(Lt,ns,ds);

%-------------------------------------------------------------------------%
%                   Numerical approximation errors                        %
%-------------------------------------------------------------------------%
vs_hat = reshape(2*Shat_*Lss,[ns ds_]);
v1_hat = vs_hat(:,1);
vk_hat = vs_hat(:,2);
vss_hat= 2*Lss;
vkk_hat= vss_hat(2,2);
for i=1:ns(1)
    resid(i,1) = -rho*v_hat(i,1) + ((c_hat(i,1)*(1-l_hat(i,1))^psi)^(1-gamma))/(1-gamma) ...
        + (A*K(i)^alpha*l_hat(i,1)^(1-alpha) - delta*K(i) - c_hat(i,1))*vk_hat(i,1) ...
        + 0.5*sigmaK^2*K(i)^2*vkk_hat;
    resid_LQc(i,1) = log(abs(resid(i,1))+0.000000000001)/log(10);
end

%-------------------------------------------------------------------------%
%                            Plot results                                 %
%-------------------------------------------------------------------------%
fig = figure(1);
setmyfig_out(fig);
set(gca, 'FontSize', 14)
subplot(3,1,1)
plot(K/Kss+1,v_hat)
hold on
plot(K/Kss+1,Vt,'r')
hold off
ylabel('$V(K_{t},\bar{A})$','FontSize',16,'Interpreter','latex')
title('Value function','FontSize',16,'Interpreter','latex')
rect =  [0.73 0.7 0.1 0.08];
h2 = legend('Approximated policy function','True policy function');
set(h2, 'Location', 'Southeast','FontSize',14,'interpreter','latex')
legend boxoff 
axis([-inf inf -inf inf])
grid on

subplot(3,1,2)
plot(K/Kss+1,c_hat)
hold on
plot(K/Kss+1,Ct,'r')
title('Consumption Function','FontSize',16,'Interpreter','latex')
ylabel('$C(K_{t},\bar{A})$','FontSize',16,'Interpreter','latex')
hold off
axis([-inf inf -inf inf])
grid on

subplot(3,1,3)
plot(K/Kss+1,l_hat)
hold on
plot(K/Kss+1,Lt,'r')
hold off
title('Labour Supply Function','FontSize',16,'Interpreter','latex')
ylabel('$L(K_{t},\bar{A})$','FontSize',16,'Interpreter','latex')
xlabel('$K/Kss$','FontSize',16,'Interpreter','latex')
axis([-inf inf -inf inf])
grid on
