%% A comparison of numerical methods for the solution of continuous-time DSGE models
%  Author: Juan Carlos Parra-Alvarez
%  Aarhus University
%  September 2013
%  Updated -> February 2016
%
%  Projection method: Collocation and Least Squares methods
%
%  Description of file: mainProj_closed.m computes the approximated value
%  and policy funtions for the RBC model under proposition 3.1 in the paper.
%  The method uses two approximants. One for the value function and other
%  for the policy function associated with labor supply. Once both 
%  approximants are computed, the consumption function is recovered from 
%  the first order conditions of the model.
%
%  To choose the type of approximant change the value of the variable
%  method in line 117:
%                       method = 1 -> Collocation method
%                       method = 2 -> Least squares method
%
%  To run the code it is necessary to have CompEcon toolbox installed in
%  the root of Matlab.
%
%  To load the steady state of the model, you need to run first the file
%  SteadyState_closed.m found in the Steady State folder. Obs: Check that 
%  the model parameters section coincidence between files.
%
%  This code uses the true solution as initial value for the numerical
%  procedures. 
%
%  The external function fun_proj.m defines the projection the residual 
%  functions. 

%% ---------------------------------------------------------------------- %
%                             House keeping                               %
%-------------------------------------------------------------------------%
clear
clc
close all

%% ---------------------------------------------------------------------- %
%                            Model Parameters                             %
%-------------------------------------------------------------------------%
global rho gamma alpha delta sigma psi A;

A     = 1;
gamma = 3.85;
psi   = 1.8011;
delta = 0.0196;
alpha = 0.4;
sigma = 0.001;
rho   = -(1-alpha*gamma)*(delta+0.5*alpha*gamma*sigma^2);

%% ---------------------------------------------------------------------- %
%                    Deterministic Steady State                           %
%-------------------------------------------------------------------------%
% Obs: Check that the model parameters section coincidence between files.
load('/Users/Juan.Carlos.Parra.Alvarez/Dropbox/Submission_MD/OnlineCodes/SteadyState/steadystate_closed.mat');
Kss  = ss_sol(1);
lss  = ss_sol(2);
css  = ss_sol(3);
vss  = ss_sol(4);
vkss = ss_sol(5);
rss  = ss_sol(6);
wss  = ss_sol(7);
clear('ss_sol');

%% ---------------------------------------------------------------------- %
%               Numerical approximation: Collocation method               %
%-------------------------------------------------------------------------%
tic;
%-------------------------------------------------------------------------%
%                   Function approximation parameters                     %
%-------------------------------------------------------------------------%
n      = 17;                              % Order of approximation
smin   = 0.1*Kss;                         % Lower bound on the state variable 
smax   = 1.5*Kss;                         % Upper bound on the state variable 
fspace = fundefn('cheb',n,smin,smax);     % Definition of basis functions
d      = fspace.d;                        % Number of state variables

%-------------------------------------------------------------------------%
%                    Chebychev nodes in [smin, smax]                      %
%-------------------------------------------------------------------------%
snodes  = funnode(fspace);
snodes1 = gridmake(snodes);
K       = snodes1;

%-------------------------------------------------------------------------%
%                      Chebyshev Basis functions                          %
%-------------------------------------------------------------------------%
bases= funbasx(fspace,snodes,[0;1;2]);
Phi  = ctbasemake([],bases,0);          % Chebyshev polynomials
Phi1 = ctbasemake([],bases,1);          % First order derivative of the Chebyshev polynomials
Phi2 = ctbasemake([],bases,2);          % Second order derivative of the Chebyshev polynomials

%-------------------------------------------------------------------------%
%                  Closed form solution (proposition 3.1)                 %
%-------------------------------------------------------------------------%
Lt = gamma*(1-alpha)/(gamma*(1-alpha)-psi*(1-gamma));
s  = 1-(1-alpha)*(1-Lt)/(psi*Lt);
C1 = -((1-s)^(1-gamma)*A^(1-gamma)*Lt^((1-alpha)*(1-gamma))*(1-Lt)^(psi*(1-gamma)))/((1-gamma)*A*Lt^(1-alpha)-(1-gamma)*(1-s)*A*Lt^(1-alpha));
v  = C1*((snodes.^(1-alpha*gamma))/(1-alpha*gamma));
Ct = (1-s)*A*snodes.^alpha*Lt^(1-alpha);

%-------------------------------------------------------------------------%
%              Initial guess for approximation coefficients               %
%-------------------------------------------------------------------------%
coeffiL = ckronxi(bases.vals(1,:),ones(n,1)*Lt,d:-1:1);
coeffiV = ckronxi(bases.vals(1,:),v,d:-1:1);
a       = coeffiV;
b       = coeffiL;
coeffi  = [a;b];

%-------------------------------------------------------------------------%
%                    Projection algorithm -> p(x)                         %
%-------------------------------------------------------------------------%
tol    = sqrt(eps);
algo   = 'trust-region-dogleg';
method = 1;
switch method
    case 1 % Collocation method
        options      = optimset('Display','iter','MaxIter', 50000, 'MaxFunEvals', 50000, 'TolFun', tol,'Algorithm', algo, 'Jacobian', 'off');
        coeff        = fsolve(@(coeff) fun_proj(Phi,Phi1,Phi2,snodes1,method,coeff), coeffi, options);
        options      = optimset('Display','iter','MaxIter', 50000, 'MaxFunEvals', 50000, 'TolFun', tol, 'Jacobian', 'off');
        [coeff,fval] = fsolve(@(coeff) fun_proj(Phi,Phi1,Phi2,snodes1,method,coeff), coeff, options);
                
    case 2 % Least Squares method
        options      = optimset('Display','iter','MaxIter', 50000, 'MaxFunEvals', 50000, 'TolFun', tol,'Algorithm', algo, 'Jacobian', 'off');
        [coeff,fval] = fminsearch(@(coeff) fun_proj(Phi,Phi1,Phi2,snodes1,method,coeff), coeffi, options);
end
toc;

%-------------------------------------------------------------------------%
%                            Evaluation grid                              %
%-------------------------------------------------------------------------%
l = size(coeff,1);
a = coeff(1:l/2);
b = coeff((l/2)+1:end);

nres = 1000; %Number of extra evaluation points over the number of grid points
for i=1:d
    n(i) = nres;
    scoord{i} = linspace(fspace.a(i),fspace.b(i),n(i))'; %Evaluation points at each state.
end
S = gridmake(scoord);
K = S;

% Basis functions on the evaluation grid
bases = funbasx(fspace,scoord,[0;1;2]);
Phi   = ctbasemake([],bases,0);
Phi1  = ctbasemake([],bases,1);
Phi2  = ctbasemake([],bases,2);

% Value function approximations
V   = Phi*a;
Vk  = Phi1{1}*a;
Vkk = Phi2{1}*a;

% Labor supply approximation
L = Phi*b;

% Optimal prices
r = alpha*A.*(K.^(alpha-1)).*(L.^(1-alpha));
w = (1-alpha)*A.*(K.^alpha).*(L.^(-alpha));

% Consumption approximation
C = (Vk.*(1-L).^(-psi*(1-gamma))).^(-1/gamma);

V  = reshape(V,[n 1]);
Vk = reshape(Vk,[n 1]);
C  = reshape(C,[n size(C,2) 1]);
L  = reshape(L,[n size(L,2) 1]);

%-------------------------------------------------------------------------%
%                   Numerical approximation errors                        %
%-------------------------------------------------------------------------%
if method == 1
    resid = -rho*V + ((C.*(1-L).^psi).^(1-gamma))./(1-gamma) + (A*K.^alpha.*L.^(1-alpha) - delta*K - C).*Vk + 0.5*sigma^2*K.^2.*Vkk;
    resid_col = log(abs(resid)+0.000000000001)/log(10);
else
    resid = -rho*V + ((C.*(1-L).^psi).^(1-gamma))./(1-gamma) + (A*K.^alpha.*L.^(1-alpha) - delta*K - C).*Vk + 0.5*sigma^2*K.^2.*Vkk;
    resid_ols = log(abs(resid)+0.000000000001)/log(10);
end

%-------------------------------------------------------------------------%
%                            Plot results                                 %
%-------------------------------------------------------------------------%
%                     True values at evaluation grid                      %
%-------------------------------------------------------------------------%
Lt = gamma*(1-alpha)/(gamma*(1-alpha)-psi*(1-gamma));
s  = 1-(1-alpha)*(1-Lt)/(psi*Lt);
C1 = -((1-s)^(1-gamma)*A^(1-gamma)*Lt^((1-alpha)*(1-gamma))*(1-Lt)^(psi*(1-gamma)))/((1-gamma)*A*Lt^(1-alpha)-(1-gamma)*(1-s)*A*Lt^(1-alpha));
v  = C1*((K.^(1-alpha*gamma))/(1-alpha*gamma));
Ct = (1-s)*A*K.^alpha*Lt^(1-alpha);

%-------------------------------------------------------------------------%
%                       Value and policy functions                        %
%-------------------------------------------------------------------------%
fig = figure(1);
setmyfig_out(fig);
set(gca, 'FontSize', 14)
subplot(3,1,1)
plot(K/Kss,V)
hold on
plot(K/Kss,v,'r')
ylabel('$V(K_{t},\bar{A})$','FontSize',16,'Interpreter','latex')
title('Value function','FontSize',16,'Interpreter','latex')
hold off
rect =  [0.73 0.7 0.1 0.08];
h2 = legend('Approximated policy function','True policy function');
set(h2, 'Location', 'Southeast','FontSize',14,'interpreter','latex')
legend boxoff 
axis([-inf inf -inf inf])
grid on

subplot(3,1,2)
plot(K/Kss,C)
hold on
plot(K/Kss,Ct,'r')
title('Consumption Function','FontSize',16,'Interpreter','latex')
ylabel('$C(K_{t},\bar{A})$','FontSize',16,'Interpreter','latex')
hold off
axis([-inf inf -inf inf])
grid on

subplot(3,1,3)
plot(K/Kss,L)
hold on
plot(K/Kss,ones(n,1)*Lt,'r')
title('Labour Supply Function','FontSize',16,'Interpreter','latex')
ylabel('$L(K_{t},\bar{A})$','FontSize',16,'Interpreter','latex')
xlabel('$K/Kss$','FontSize',16,'Interpreter','latex')
hold off
axis([-inf inf lss*0.9999 lss*1.0001])
grid on