%% A comparison of numerical methods for the solution of continuous-time DSGE models
%  Author: Juan Carlos Parra-Alvarez
%  Aarhus University
%  September 2013
%
%  Deterministic steady state for model.
%  This files calls the function ss_bench.m
%
%  The deterministic steady state can be computed analitically. However,
%  this file solves numerically for the deterministic steady state of the 
%  model using user provided initial values. 
%
%  Remember to check the parameter values such that they coincide with
%  mainProj_Bench.m

%% ---------------------------------------------------------------------- %
%                             House keeping                               %
%-------------------------------------------------------------------------%
clear
clc
close all

%% ---------------------------------------------------------------------- %
%                               Parameters                                %
%-------------------------------------------------------------------------%
global gamma psi delta rho alpha kappa omega;

gamma = 2;
psi   = 1.8011;
delta = 0.0196;
rho   = 0.0105;
alpha = 0.4;
kappa = 0.05;
omega = 1;
sigma = 0.000;
eta   = 0.007;

%% ---------------------------------------------------------------------- %
%                              Steady State                               %
%-------------------------------------------------------------------------%

%-------------------------------------------------------------------------%
%                            Initial values                               %
%-------------------------------------------------------------------------%

% Analytical steady state 
A  = omega;
l  = (1-alpha)/(1+psi-alpha-(alpha*psi*delta/(rho+delta)));
k  = (alpha*A/(rho+delta))^(1/(1-alpha))*l;
r  = rho+delta;
w  = (1-alpha)*A*k^alpha*l^(-alpha);
c  = (r-delta)*k+w*l;
v  = (c*(1-l)^psi)^(1-gamma)/(rho*(1-gamma));
vk = c^(-gamma)*(1-l)^(psi*(1-gamma));
vA = (r-delta)*vk/(kappa+rho);

% A  = 1;
% l  = 0.3398;
% k  = 30;
% r  = 1;
% w  = 1;
% c  = 0.611;
% v  = -328;
% vk = 5;
% vA = 1;

X0 = [A;k;l;c;v;vk;vA;r;w];

%-------------------------------------------------------------------------%
%                        Algorithm parameters                             %
%-------------------------------------------------------------------------%
algo    = 'trust-region-reflective';
options = optimset('Display','iter','MaxIter', 50000,'MaxFunEvals', 50000,'Algorithm', algo);

%-------------------------------------------------------------------------%
%                        Model's steady state                             %
%-------------------------------------------------------------------------%
ss_sol = fsolve(@(X) ss_bench(X),X0,options);

A  = ss_sol(1);
k  = ss_sol(2);
l  = ss_sol(3);
c  = ss_sol(4);
v  = ss_sol(5);
vk = ss_sol(6);
vA = ss_sol(7);
r  = ss_sol(8);
w  = ss_sol(9);

save('steadystate.mat','ss_sol')
