%% A comparison of numerical methods for the solution of continuous-time DSGE models
%  Author: Juan Carlos Parra-Alvarez
%  Aarhus University
%  September 2013
%
%  Deterministic steady state for model under prop. 3.2. 
%  This files calls the function ss_closed.m
%
%  The deterministic steady state can be computed analitically. However,
%  this file solves numerically for the deterministic steady state of the 
%  model using user provided initial values. 
%
%  Remember to check the parameter values such that they coincide with
%  mainProj_closed.m

%% ---------------------------------------------------------------------- %
%                             House keeping                               %
%-------------------------------------------------------------------------%
clear
clc
close all

%% ---------------------------------------------------------------------- %
%                               Parameters                                %
%-------------------------------------------------------------------------%
global gamma psi delta rho alpha A;

A     = 1;
gamma = 3.85;
psi   = 1.8011;
delta = 0.0196;
alpha = 0.4;
sigma = 0.001;
rho   = -(1-alpha*gamma)*(delta+0.5*alpha*gamma*sigma^2);

%% ---------------------------------------------------------------------- %
%                              Steady State                               %
%-------------------------------------------------------------------------%

%-------------------------------------------------------------------------%
%                            Initial values                               %
%-------------------------------------------------------------------------%

% Analytical steady state
% A  = omega;
% l  = (1-alpha)/(1+psi-alpha-(alpha*psi*delta/(rho+delta)));
% k  = (alpha*A/(rho+delta))^(1/(1-alpha))*l;
% r  = rho+delta;
% w  = (1-alpha)*A*k^alpha*l^(-alpha);
% c  = (r-delta)*k+w*l;
% v  = (c*(1-l)^psi)^(1-gamma)/(rho*(1-gamma));
% vk = c^(-gamma)*(1-l)^(psi*(1-gamma));
% vA = (r-delta)*vk/(kappa+rho);

l  = 0.3398;
k  = 4.3;
r  = 1;
w  = 1;
c  = 0.611;
v  = -328;
vk = 5;

X0 = [k;l;c;v;vk;r;w];

%-------------------------------------------------------------------------%
%                        Algorithm parameters                             %
%-------------------------------------------------------------------------%
algo    = 'trust-region-reflective';
options = optimset('Display','iter','MaxIter', 50000,'MaxFunEvals', 50000,'Algorithm', algo);

%-------------------------------------------------------------------------%
%                        Model's steady state                             %
%-------------------------------------------------------------------------%
ss_sol = fsolve(@(X) ss_closed(X),X0,options);

k  = ss_sol(1);
l  = ss_sol(2);
c  = ss_sol(3);
v  = ss_sol(4);
vk = ss_sol(5);
r  = ss_sol(6);
w  = ss_sol(7);

save('steadystate_closed.mat','ss_sol')
